@@ -27,7 +27,7 @@
 其中result.info接口可以传record扩展信息，比如::
 
     result.error('', '异常发生', traceback=traceback.format_exc())
-        
+
 同logger一样，TestResult对象保证对所有的ITestResultHandler的调用都是线程安全的，可以通过实现
 ITestResultHandler来实现一个新的Handler，详细请参考ITestResultHandler接口
 
@@ -72,6 +72,11 @@ class EnumLogLevel(object):
     TESTTIMEOUT = 62  # 测试执行超时
     RESNOTREADY = 69  # 当前资源不能满足测试执行的要求
 
+class TestResultType(object):
+    '''扩展用例状态
+    '''
+    FILTERED = 'ignored'
+
 
 levelname = {}
 for name in EnumLogLevel.__dict__:
@@ -79,6 +84,11 @@ for name in EnumLogLevel.__dict__:
     if isinstance(value, int):
         levelname[value] = name
 
+RESULT_TYPES = (
+    (TestResultType.FILTERED, "被忽略"),
+)
+
+
 
 def _convert_timelength(sec):
     h = int(sec / 3600)
@@ -99,7 +109,7 @@ def smart_text_by_lines(s):
 
 class TestResultBase(object):
     '''测试结果基类
-    
+
     此类的职责如下：
     1、提供测试结果基本接口
     2、保证线程安全
@@ -119,6 +129,7 @@ class TestResultBase(object):
         self.__error_level = 0
         self.__failed_info = ""
         self.__failed_priority = 0
+        self._custom_result = None
 
     @property
     def testcase(self):
@@ -130,7 +141,7 @@ class TestResultBase(object):
     @property
     def passed(self):
         '''测试是否通过
-        
+
         :returns: True or False
         '''
         return all(self.__steps_passed)
@@ -138,7 +149,7 @@ class TestResultBase(object):
     @property
     def failed_reason(self):
         '''用例测试不通过的错误原因
-        
+
         :returns: str
         '''
         if self.__error_level:
@@ -149,7 +160,7 @@ class TestResultBase(object):
     @property
     def failed_info(self):
         '''测试用例失败时的执行状况
-        
+
         :returns: str
         '''
         return self.__failed_info
@@ -157,7 +168,7 @@ class TestResultBase(object):
     @property
     def begin_time(self):
         '''测试用例开始时间
-        
+
         :returns: float
         '''
         return self.__begin_time
@@ -165,14 +176,14 @@ class TestResultBase(object):
     @property
     def end_time(self):
         '''测试用例结束时间
-        
+
         :returns: float
         '''
         return self.__end_time
 
     def begin_test(self, testcase):
         '''开始执行测试用例
-        
+
         :param testcase: 测试用例
         :type testcase: TestCase
         '''
@@ -197,7 +208,7 @@ class TestResultBase(object):
 
     def begin_step(self, msg):
         '''开始一个测试步骤
-        
+
         :param msg: 测试步骤名称
         :type msg: string
         '''
@@ -212,7 +223,7 @@ class TestResultBase(object):
 
     def log_record(self, level, msg, record=None, attachments=None):
         '''处理一个日志记录
-        
+
         :param level: 日志级别，参考EnumLogLevel
         :type level: string
         :param msg: 日志消息
@@ -325,7 +336,7 @@ class TestResultBase(object):
 
     def handle_test_begin(self, testcase):
         '''处理一个测试用例执行的开始
-        
+
         :param testcase: 测试用例
         :type testcase: TestCase
         '''
@@ -333,7 +344,7 @@ class TestResultBase(object):
 
     def handle_test_end(self, passed):
         '''处理一个测试用例执行的结束
-        
+
         :param passed: 测试用例是否通过
         :type passed: boolean
         '''
@@ -341,7 +352,7 @@ class TestResultBase(object):
 
     def handle_step_begin(self, msg):
         '''处理一个测试步骤的开始
-        
+
         :param msg: 测试步骤名称
         :type msg: string
         '''
@@ -349,7 +360,7 @@ class TestResultBase(object):
 
     def handle_step_end(self, passed):
         '''处理一个测试步骤的结束
-        
+
         :param passed: 测试步骤是否通过
         :type passed: boolean
         '''
@@ -357,7 +368,7 @@ class TestResultBase(object):
 
     def handle_log_record(self, level, msg, record, attachments):
         '''处理一个日志记录
-        
+
         :param level: 日志级别，参考EnumLogLevel
         :type level: string
         :param msg: 日志消息
@@ -369,6 +380,8 @@ class TestResultBase(object):
         '''
         pass
 
+    def customize_result(self, result):
+        self._custom_result = result
 
 class EmptyResult(TestResultBase):
     '''不输出
@@ -385,7 +398,7 @@ class StreamResult(TestResultBase):
 
     def __init__(self, stream=sys.stdout):
         '''构造函数
-        
+
         :param stream: 流对象
         :type stream: file
         '''
@@ -396,7 +409,7 @@ class StreamResult(TestResultBase):
 
     def handle_test_begin(self, testcase):
         '''处理一个测试用例执行的开始
-        
+
         :param testcase: 测试用例
         :type testcase: TestCase
         '''
@@ -410,7 +423,7 @@ class StreamResult(TestResultBase):
 
     def handle_test_end(self, passed):
         '''处理一个测试用例执行的结束
-        
+
         :param passed: 测试用例是否通过
         :type passed: boolean
         '''
@@ -420,16 +433,19 @@ class StreamResult(TestResultBase):
         self._write("测试用例执行时间: %02d:%02d:%02.2f\n" % _convert_timelength(self.end_time - self.begin_time))
 
         rsttxts = {True:'通过', False:'失败'}
+        test_result = rsttxts[passed]
+        if self._custom_result:
+            test_result = dict(RESULT_TYPES)[self._custom_result]
         steptxt = ''
         for i, ipassed in enumerate(self._step_results):
             steptxt += " %s:%s" % (i + 1, rsttxts[ipassed])
         self._write("测试用例步骤结果: %s\n" % steptxt)
-        self._write("测试用例最终结果: %s\n" % rsttxts[passed])
+        self._write("测试用例最终结果: %s\n" % test_result)
         self._write(self._seperator2)
 
     def handle_step_begin(self, msg):
         '''处理一个测试步骤的开始
-        
+
         :param msg: 测试步骤名称
         :type msg: string
         '''
@@ -441,7 +457,7 @@ class StreamResult(TestResultBase):
 
     def handle_step_end(self, passed):
         '''处理一个测试步骤的结束
-        
+
         :param passed: 测试步骤是否通过
         :type passed: boolean
         '''
@@ -449,7 +465,7 @@ class StreamResult(TestResultBase):
 
     def handle_log_record(self, level, msg, record, attachments):
         '''处理一个日志记录
-        
+
         :param level: 日志级别，参考EnumLogLevel
         :type level: string
         :param msg: 日志消息
@@ -487,14 +503,14 @@ class XmlResult(TestResultBase):
 
     def __init__(self, testcase):
         '''构造函数
-        
+
         :param file_path: XML文件路径
         :type file_path: string
         '''
         super(XmlResult, self).__init__()
         self._xmldoc = dom.Document()
         translated_name = translate_bad_char(testcase.test_name)
-        max_name_len = 240
+        max_name_len = 200
         if len(translated_name) > max_name_len:
             translated_name = translated_name[:max_name_len]
         self._file_path = '%s_%s.xml' % (translated_name, get_time_str())
@@ -502,14 +518,14 @@ class XmlResult(TestResultBase):
     @property
     def file_path(self):
         '''xml文件路径
-        
+
         :returns: str
         '''
         return self._file_path
 
     def handle_test_begin(self, testcase):
         '''处理一个测试用例执行的开始
-        
+
         :param testcase: 测试用例
         :type testcase: TestCase
         '''
@@ -523,17 +539,23 @@ class XmlResult(TestResultBase):
         self._testnode.setAttribute("priority", str(priority))
         self._testnode.setAttribute("timeout", str(timeout))
         self._testnode.setAttribute('begintime', time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.begin_time)))
+        extra_properties = testcase.get_test_extra_properties()
+        for k, v in extra_properties.items():
+            self._testnode.setAttribute(k, str(v))
         self._xmldoc.appendChild(self._testnode)
 
         self.begin_step('测试用例初始步骤')
 
     def handle_test_end(self, passed):
         '''处理一个测试用例执行的结束
-        
+
         :param passed: 测试用例是否通过
         :type passed: boolean
         '''
-        self._testnode.setAttribute('result', str(passed))
+        test_result = str(passed)
+        if self._custom_result:
+            test_result = str(self._custom_result)
+        self._testnode.setAttribute('result', test_result)
         self._testnode.setAttribute('endtime', time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.end_time)))
         self._testnode.setAttribute('duration', "%02d:%02d:%02.2f\n" % _convert_timelength(self.end_time - self.begin_time))
         if self._file_path:
@@ -542,7 +564,7 @@ class XmlResult(TestResultBase):
 
     def handle_step_begin(self, msg):
         '''处理一个测试步骤的开始
-        
+
         :param msg: 测试步骤名称
         :type msg: string
         '''
@@ -555,7 +577,7 @@ class XmlResult(TestResultBase):
 
     def handle_step_end(self, passed):
         '''处理一个测试步骤的结束
-        
+
         :param passed: 测试步骤是否通过
         :type passed: boolean
         '''
@@ -563,7 +585,7 @@ class XmlResult(TestResultBase):
 
     def handle_log_record(self, level, msg, record, attachments):
         '''处理一个日志记录
-        
+
         :param level: 日志级别，参考EnumLogLevel
         :type level: string
         :param msg: 日志消息
@@ -635,7 +657,7 @@ class XmlResult(TestResultBase):
 
     def toxml(self):
         '''返回xml文本
-        
+
         :returns string - xml文本
         '''
         return to_pretty_xml(self._xmldoc)
@@ -668,7 +690,7 @@ class JSONResult(TestResultBase):
 
     def handle_test_begin(self, testcase):
         '''处理一个测试用例执行的开始
-        
+
         :param testcase: 测试用例
         :type testcase: TestCase
         '''
@@ -676,10 +698,15 @@ class JSONResult(TestResultBase):
 
     def handle_test_end(self, passed):
         '''处理一个测试用例执行的结束
-        
+
         :param passed: 测试用例是否通过
         :type passed: boolean
         '''
+        rsttxts = {True:'通过', False:'失败'}
+        test_result = rsttxts[passed]
+        if self._custom_result:
+            test_result = dict(RESULT_TYPES)[self._custom_result]
+        self._data["result_type"] = test_result
         self._data["succeed"] = passed
         self._data["start_time"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.begin_time))
         self._data["end_time"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.end_time))
@@ -687,7 +714,7 @@ class JSONResult(TestResultBase):
 
     def handle_step_begin(self, msg):
         '''处理一个测试步骤的开始
-        
+
         :param msg: 测试步骤名称
         :type msg: string
         '''
@@ -699,7 +726,7 @@ class JSONResult(TestResultBase):
 
     def handle_step_end(self, passed):
         '''处理一个测试步骤的结束
-        
+
         :param passed: 测试步骤是否通过
         :type passed: boolean
         '''
@@ -709,7 +736,7 @@ class JSONResult(TestResultBase):
 
     def handle_log_record(self, level, msg, record, attachments):
         '''处理一个日志记录
-        
+
         :param level: 日志级别，参考EnumLogLevel
         :type level: string
         :param msg: 日志消息
@@ -752,7 +779,7 @@ class TestResultCollection(list):
 
     def __init__(self, results, passed):
         '''构造函数
-        
+
         :param results: 测试结果列表
         :type results: list
         :param passed: 测试是否通过
@@ -764,7 +791,7 @@ class TestResultCollection(list):
     @property
     def passed(self):
         '''测试是否通过
-        
+
         :returns: boolean
         '''
         return self.__passed
